package amidagame;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Random;

import networking.packets.*;

import amidagame.packets.PackageAmidaAck;
import amidagame.packets.PackageAmidaGameOver;
import amidagame.packets.PackageAmidaLogIn;
import amidagame.packets.PackageAmidaLogInAccepted;
import amidagame.packets.PackageCoinInfo;
import amidagame.packets.PackageCoinReq;
import amidagame.packets.PackageDrawLineReq;
import amidagame.packets.PackageNewLines;
import amidagame.packets.PackageUnregisterPlayer;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryonet.Connection;
import com.esotericsoftware.kryonet.Server;
import com.esotericsoftware.minlog.Log;

public class AmidaServer {
	private Server server;
	public LinkedList<Connection> cList;
	public LinkedList<Connection> cList2;
	private int count;
	private LinkedList<Float> queueX;
	private LinkedList<Float> queueY;
	private LinkedList<Short> queuePid;
	private Random rand;
	
	boolean[] playersId;
	int[] coinsLines;
	float[] coinsYCords;
	int[] boxes;
	
	// Hash Maps
	// Keys are all game ids
	HashMap<Integer,GameInfo> coinInfoMap;
	HashMap<Integer, HashMap<Short, LinkedList<LineInfo>>> newLineMap;
	HashMap<Integer,HashMap<Short,Boolean>> changesMap;
	
	private LinkedList<Short> gameQueue;
	public boolean connecting;

	public AmidaServer() throws IOException {
		rand = new Random();
		rand.setSeed(System.nanoTime());
		server = new Server();
		queueX = new LinkedList<Float>();
		queueY = new LinkedList<Float>();
		queuePid = new LinkedList<Short>();
		gameQueue = new LinkedList<Short>();
		
		playersId = new boolean[4];
		coinsLines = new int[6];
		coinsYCords = new float[6];
		boxes = new int[4];
		resetGame();
		
		coinInfoMap = new HashMap<Integer, GameInfo>();
		newLineMap = new HashMap<Integer, HashMap<Short,LinkedList<LineInfo>>>();
		changesMap = new HashMap<Integer,HashMap<Short,Boolean>>();

		cList = new LinkedList<Connection>();
		cList2 = new LinkedList<Connection>();
		count = 0;
		registerPackets();
		// TODO
		AmidaListenerS l = new AmidaListenerS(this);
		server.addListener(l);
		server.bind(13131);
		server.start();
	}

	private void registerPackets() {
		Kryo kryo = server.getKryo();
		kryo.register(Packet0000LogInfo.class);
		kryo.register(Packet0001LogInAccepted.class);
		kryo.register(Packet0002Message.class);
		kryo.register(Packet0003SyncReq.class);
		kryo.register(Packet0004Ack.class);
		kryo.register(PackageAmidaGameOver.class);
		kryo.register(PackageAmidaLogIn.class);
		kryo.register(PackageAmidaLogInAccepted.class);
		kryo.register(PackageAmidaAck.class);
		kryo.register(PackageDrawLineReq.class);
		kryo.register(PackageNewLines.class);
		kryo.register(PackageUnregisterPlayer.class);
		kryo.register(PackageCoinInfo.class);
		kryo.register(PackageCoinReq.class);
		kryo.register(short[].class);
		kryo.register(float[].class);
		kryo.register(int[].class);
	}

	public int getCount() {
		return count;
	}

	public void setCount(int count) {
		this.count = count;
	}

	public static void main(String[] args){
		try {
			new AmidaServer();
			Log.set(Log.LEVEL_DEBUG);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public LinkedList<Float> getQueueX() {
		return queueX;
	}

	public void setQueueX(LinkedList<Float> queueX) {
		this.queueX = queueX;
	}

	public LinkedList<Float> getQueueY() {
		return queueY;
	}

	public void setQueueY(LinkedList<Float> queueY) {
		this.queueY = queueY;
	}

	public LinkedList<Short> getQueuePid() {
		return queuePid;
	}

	public void setQueuePid(LinkedList<Short> queuePid) {
		this.queuePid = queuePid;
	}

	public void addNewLine(float x, float y, short pid) {
		// TODO Auto-generated method stub
		queueX.offer(x);
		queueY.offer(y);
		queuePid.offer(pid);
	}

	public float[] takeAllX() {
		// TODO Auto-generated method stub
		float[] result = new float[queueX.size()];
		int index = 0;
		
		while(!queueX.isEmpty()){
			result[index] = queueX.poll();
			index++;
		}
		
		return result;
	}
	
	public float[] takeAllY() {
		// TODO Auto-generated method stub
		float[] result = new float[queueY.size()];
		int index = 0;
		
		while(!queueY.isEmpty()){
			result[index] = queueY.poll();
			index++;
		}
		
		return result;
	}
	
	public short[] takeAllPid() {
		// TODO Auto-generated method stub
		
		short[] result = new short[queuePid.size()];
		int index = 0;
		
		while(!queuePid.isEmpty()){
			result[index] = queuePid.poll();
			index++;
		}
		
		return result;
	}
	
	public short assignNewPid(){
		return assignNewPid(-1);
	}

	public short assignNewPid(int gameid) {
		if (gameid < 0){
			for (short i = 0; i < playersId.length; i++){
				if (!playersId[i]){
					playersId[i] = true;
					return i;
				}
			}
		} else {
			boolean[] idlist = coinInfoMap.get(gameid).playersId;
			for (short i = 0; i < idlist.length; i++){
				if (!idlist[i]){
					coinInfoMap.get(gameid).playersId[i] = true;
					newLineMap.get(gameid).put(i, new LinkedList<LineInfo>());
					changesMap.get(gameid).put(i, false);
					return i;
				}
			}
		}
		
		// MORE MANAGEMENT LATER
		
		return -1;
	}
	
	public void registerPid(short myPid){
		playersId[myPid] = true;
	}
	
	public void unRegisterPid(short myPid){
		playersId[myPid] = false;
	}
	
	public GameInfo resetGame(){
		rand.setSeed(System.nanoTime());
		
		GameInfo result = new GameInfo();
		
		LinkedList<Integer> coinList = new LinkedList<Integer>();
		for (int i = 0; i < 4; i++){
			playersId[i] = false;
			coinList.add(i + 1);
		}
		Collections.shuffle(coinList);
		for (int i = 0; i < 4; i++){
			boxes[i] = coinList.get(i);
			System.out.println("Box at line " + i + " is: " + boxes[i]);
		}
		for (int i = 0; i < 6; i++){
			coinsLines[i] = rand.nextInt(4);
			coinsYCords[i] = rand.nextFloat()*0.7f+0.1f;
			System.out.println("Random new Coin at Line " + coinsLines[i] + " And YCord is: " + coinsYCords[i]);
		}
		result.playersId = playersId.clone();
		result.boxes = boxes.clone();
		result.coinsLines = coinsLines.clone();
		result.coinsYCords = coinsYCords.clone();
		
		return result;
	}

	public void broadcastNewLine(PackageDrawLineReq r) {
		// TODO Auto-generated method stub
		//server.sendToAllTCP(r);
		if (r.gameid>=0){
			newLineMap.get(r.gameid).get(r.pid).add(new LineInfo(r.x,r.y,r.pid));
			changesMap.get(r.gameid).put(r.pid, true);
		} else {
			server.sendToAllTCP(r);
		}
	}
	
	public void boardcastNewLines(Connection c, short pid, int gameid){
		if (gameid>=0){
			if (changesMap.get(gameid).get(pid)){			
				int newLines = newLineMap.get(gameid).size();
				PackageNewLines p = new PackageNewLines();
				p.pid = new short[newLines];
				p.x = new float[newLines];
				p.y = new float[newLines];
				int i = 0;
				
				for (LineInfo l: newLineMap.get(gameid).get(pid)){
					p.pid[i] = l.pid;
					p.x[i] = l.x;
					p.y[i] = l.y;
					
					i++;
				}
				
				c.sendTCP(p);
				finishChange(gameid, pid);
			}
		}
	}
	
	
	public void finishChange(int gameid, short pid){
		newLineMap.get(gameid).put(pid, new LinkedList<LineInfo>());
		changesMap.get(gameid).put(pid, false);
	}

	public void drawCoins(Connection arg0, PackageCoinReq arg1) {
		// TODO Auto-generated method stub
		PackageCoinInfo p = new PackageCoinInfo();
		if (coinInfoMap.get(arg1.gameid)!=null){
			p.lines = coinInfoMap.get(arg1.gameid).coinsLines.clone();
			p.yCords = coinInfoMap.get(arg1.gameid).coinsYCords.clone();
			p.boxes = coinInfoMap.get(arg1.gameid).boxes.clone();
		} else {
			p.lines = coinsLines.clone();
			p.yCords = coinsYCords.clone();
			p.boxes = boxes.clone();
		}
		arg0.sendTCP(p);
	}

	public void createNewGame(int gameid) {
		// TODO Auto-generated method stub
		GameInfo newgame = resetGame();
		coinInfoMap.put(gameid, newgame);
	}
}
