package creaturegame;

import org.andengine.entity.scene.Scene;
import org.andengine.entity.scene.background.Background;
import org.andengine.entity.scene.background.SpriteBackground;
import org.andengine.entity.sprite.AnimatedSprite;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.font.FontFactory;
import org.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlas;
import org.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlasTextureRegionFactory;
import org.andengine.opengl.texture.region.ITextureRegion;
import org.andengine.opengl.texture.region.TiledTextureRegion;
import org.andengine.util.HorizontalAlign;

import android.content.pm.ActivityInfo;
import android.graphics.Color;
import android.graphics.Typeface;
import android.util.Log;

import com.example.traingame.SceneManager;
import com.example.traingame.Setup;

public class cWinSetup extends Setup {

	// buttons
	private BitmapTextureAtlas yesTA;
	private ITextureRegion yesTR;
	private BitmapTextureAtlas noTA;
	private ITextureRegion noTR;
	private Font mFont;
	
	private BitmapTextureAtlas backgroundTA;
	private ITextureRegion backgroundReg;
	
	private BitmapTextureAtlas backButtonTA;
	private ITextureRegion backButtonReg;

	private BitmapTextureAtlas headsTA;
	private TiledTextureRegion headsTR;
	private String headSheet = "creature creator head sheet.png";
	private String bodySheet = "creature creator body sheet.png";
	private String legSheet = "creature creator legs sheet.png";

	private BitmapTextureAtlas bodysTA;
	private TiledTextureRegion bodysTR;

	private BitmapTextureAtlas legsTA;
	private TiledTextureRegion legsTR;

	private static int H_COLUMN = 3;
	private static int H_ROWS = 2;

	private static int B_COLUMN = 3;
	private static int B_ROWS = 2;

	private static int L_COLUMN = 3;
	private static int L_ROWS = 3;

	private AnimatedSprite spritePart[] = { null, null, null };
	
	private Text topText;

	public cWinSetup(SceneManager SM) {
		this.sm = SM;
	}

	@Override
	public void load() {
		BitmapTextureAtlasTextureRegionFactory
				.setAssetBasePath("gfx/creature/");
		headsTA = new BitmapTextureAtlas(sm.activity.getTextureManager(), 2048,
				1024);
		headsTR = BitmapTextureAtlasTextureRegionFactory.createTiledFromAsset(
				headsTA, sm.activity.getAssets(), headSheet, 0, 0, H_COLUMN,
				H_ROWS);
		headsTA.load();

		bodysTA = new BitmapTextureAtlas(sm.activity.getTextureManager(), 2048,
				1024);
		bodysTR = BitmapTextureAtlasTextureRegionFactory.createTiledFromAsset(
				bodysTA, sm.activity.getAssets(), bodySheet, 0, 0, B_COLUMN,
				B_ROWS);
		bodysTA.load();

		legsTA = new BitmapTextureAtlas(sm.activity.getTextureManager(), 2048,
				2048);
		legsTR = BitmapTextureAtlasTextureRegionFactory.createTiledFromAsset(
				legsTA, sm.activity.getAssets(), legSheet, 0, 0, L_COLUMN,
				L_ROWS);
		legsTA.load();

		// font
		mFont = FontFactory.create(this.sm.engine.getFontManager(),
				this.sm.engine.getTextureManager(), 256, 256,
				Typeface.create(Typeface.DEFAULT, Typeface.BOLD), 40, true,
				Color.BLACK);
		mFont.load();
		
		backgroundTA = new BitmapTextureAtlas(sm.activity.getTextureManager(),
				2048, 2048);
		backgroundReg = BitmapTextureAtlasTextureRegionFactory.createFromAsset(
				backgroundTA, sm.activity, "generic po background.png", 0, 0);
		backgroundTA.load();

		// buttons
		yesTA = new BitmapTextureAtlas(sm.activity.getTextureManager(), 128, 64);
		yesTR = BitmapTextureAtlasTextureRegionFactory.createFromAsset(yesTA,
				sm.activity, "126x64_button.png", 0, 0);
		yesTA.load();
		noTA = new BitmapTextureAtlas(sm.activity.getTextureManager(), 128, 64);
		noTR = BitmapTextureAtlasTextureRegionFactory.createFromAsset(noTA,
				sm.activity, "126x64_button.png", 0, 0);
		noTA.load();
		
		backButtonTA = new BitmapTextureAtlas(sm.activity.getTextureManager(),
				512, 256);
		backButtonReg = BitmapTextureAtlasTextureRegionFactory.createFromAsset(
				backButtonTA, sm.activity, "back button.png", 0, 0);
		backButtonTA.load();

	}

	@Override
	public Scene create() {
		Scene cWinScene = new Scene();
		// sm.activity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		int cX = (int) (sm.camera.getWidth() - backgroundReg.getWidth()) / 2;
		int cY = (int) (sm.camera.getHeight() - backgroundReg.getHeight()) / 2;
		Sprite bgSprite = new Sprite(cX, cY, backgroundReg,
				sm.activity.getVertexBufferObjectManager());
		bgSprite.setScaleX(sm.camera.getWidth() / bgSprite.getWidth());
		bgSprite.setScaleY(sm.camera.getHeight() / bgSprite.getHeight());
		SpriteBackground bg = new SpriteBackground(bgSprite);
		cWinScene.setBackground(bg);
		
		spritePart[1] = new AnimatedSprite(0, 0, bodysTR,
				sm.activity.getVertexBufferObjectManager());
		spritePart[1].setPosition(
				(sm.camera.getWidth() - spritePart[1].getWidth()) / 2,
				(sm.camera.getHeight() - spritePart[1].getHeight()) * 4 / 10);
		cWinScene.attachChild(spritePart[1]);

		float bodyYPosition = (sm.camera.getHeight() - spritePart[1]
				.getHeight()) * 4 / 10;

		spritePart[0] = new AnimatedSprite(0, 0, headsTR,
				sm.activity.getVertexBufferObjectManager());
		spritePart[0].setPosition(
				(sm.camera.getWidth() - spritePart[0].getWidth()) / 2,
				bodyYPosition - spritePart[0].getHeight());
		cWinScene.attachChild(spritePart[0]);
		
		topText = new Text(sm.camera.getWidth() * 1 / 2 - 10,
				0, mFont, "Result", new TextOptions(HorizontalAlign.CENTER),
				this.sm.engine.getVertexBufferObjectManager());
		cWinScene.attachChild(topText);


		spritePart[2] = new AnimatedSprite(0, 0, legsTR,
				sm.activity.getVertexBufferObjectManager());

		spritePart[2].setPosition(
				(sm.camera.getWidth() - spritePart[2].getWidth()) / 2,
				bodyYPosition + spritePart[1].getHeight());
		cWinScene.attachChild(spritePart[2]);

		// buttons
		/*Sprite yesButton = new Sprite(0, 0, yesTR,
				sm.engine.getVertexBufferObjectManager()) {
			@Override
			public boolean onAreaTouched(final TouchEvent pSceneTouchEvent,
					final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				sm.setCurrentS(SceneManager.AllScenes.MENU);
				return true;
			}
		};
		yesButton.setPosition(
				(sm.camera.getWidth() * 2 / 5 - yesButton.getWidth() / 2),
				(sm.camera.getHeight() * 10 / 10 - yesButton.getHeight() / 2));
		cWinScene.attachChild(yesButton);
		cWinScene.registerTouchArea(yesButton);

		Sprite noButton = new Sprite(0, 0, noTR,
				sm.engine.getVertexBufferObjectManager()) {
			@Override
			public boolean onAreaTouched(final TouchEvent pSceneTouchEvent,
					final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				sm.setCurrentS(SceneManager.AllScenes.MENU);
				return true;
			}
		};
		noButton.setPosition(
				(sm.camera.getWidth() * 4 / 5 - noButton.getWidth() / 2),
				(sm.camera.getHeight() * 10 / 10 - noButton.getHeight() / 2));
		cWinScene.attachChild(noButton);
		cWinScene.registerTouchArea(noButton);*/
		Sprite backButton = new Sprite(100, 100, backButtonReg,
				sm.activity.getVertexBufferObjectManager()) {
			@Override
			public boolean onAreaTouched(final TouchEvent pSceneTouchEvent,
					final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				sm.activity.finish();
				return true;
			}
		};
		backButton.setScale(0.5f);
		backButton.setPosition(
				(sm.camera.getWidth() - backButton.getWidth()) / 2,
				bodyYPosition + spritePart[1].getHeight() + spritePart[2].getHeight() + 10);
		cWinScene.attachChild(backButton);
		cWinScene.registerTouchArea(backButton);

		return cWinScene;
	}

	public void setWin(int[] selections) {

		spritePart[0].setCurrentTileIndex(selections[0]);
		spritePart[1].setCurrentTileIndex(selections[1]);
		spritePart[2].setCurrentTileIndex(selections[2]);

	}

}
