package creaturegame;


import org.andengine.engine.handler.timer.ITimerCallback;
import org.andengine.engine.handler.timer.TimerHandler;
import org.andengine.entity.scene.IOnSceneTouchListener;
import org.andengine.entity.scene.Scene;
import org.andengine.entity.scene.background.Background;
import org.andengine.entity.scene.background.SpriteBackground;
import org.andengine.entity.sprite.AnimatedSprite;
import org.andengine.entity.sprite.Sprite;
import org.andengine.entity.text.Text;
import org.andengine.entity.text.TextOptions;
import org.andengine.input.touch.TouchEvent;
import org.andengine.opengl.font.Font;
import org.andengine.opengl.font.FontFactory;
import org.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlas;
import org.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlasTextureRegionFactory;
import org.andengine.opengl.texture.region.ITextureRegion;
import org.andengine.opengl.texture.region.TiledTextureRegion;
import org.andengine.util.HorizontalAlign;

import android.graphics.Color;
import android.graphics.Typeface;
import android.util.Log;

import android.content.pm.ActivityInfo;

import com.example.lctnetworktesting.networking.packets.*;
import com.example.traingame.SceneManager;
import com.example.traingame.Setup;

public class creatureSetup extends Setup implements IOnSceneTouchListener {

	private BitmapTextureAtlas headsTA;
	private TiledTextureRegion headsTR;
	private String headSheet = "creature creator head sheet v2.png";
	private String bodySheet = "creature creator body sheet v2.png";
	private String legSheet = "creature creator legs sheet v2.png";

	private BitmapTextureAtlas bodysTA;
	private TiledTextureRegion bodysTR;

	private BitmapTextureAtlas legsTA;
	private TiledTextureRegion legsTR;

	private static int H_COLUMN = 3;
	private static int H_ROWS = 2;

	private static int B_COLUMN = 3;
	private static int B_ROWS = 2;

	private static int L_COLUMN = 3;
	private static int L_ROWS = 3;

	private AnimatedSprite[] spritePart = { null, null, null };
	private int[] selections = {0, 0, 0};

	private BitmapTextureAtlas closeButtonTA;
	private ITextureRegion closeButtonReg;
	
	private BitmapTextureAtlas backgroundTA;
	private ITextureRegion backgroundReg;
	private Font mFont;
	private int timer;
	private Text timerText;
	private boolean runTimer;
	private int partId = 0;

	private int[] maxFrame = { 5, 5, 7 };
	private int[] currentImg = { 0, 0, 0 };

	private float downx = -1;
	private float downy = -1;

	public CCClient client;

	public creatureSetup(SceneManager SM) {
		this.sm = SM;
	}

	@Override
	public void load() {

		BitmapTextureAtlasTextureRegionFactory
				.setAssetBasePath("gfx/creature/");
		headsTA = new BitmapTextureAtlas(sm.activity.getTextureManager(), 2048,
				1024);
		headsTR = BitmapTextureAtlasTextureRegionFactory.createTiledFromAsset(
				headsTA, sm.activity.getAssets(), headSheet, 0, 0, H_COLUMN,
				H_ROWS);
		headsTA.load();

		bodysTA = new BitmapTextureAtlas(sm.activity.getTextureManager(), 2048,
				1024);
		bodysTR = BitmapTextureAtlasTextureRegionFactory.createTiledFromAsset(
				bodysTA, sm.activity.getAssets(), bodySheet, 0, 0, B_COLUMN,
				B_ROWS);
		bodysTA.load();

		legsTA = new BitmapTextureAtlas(sm.activity.getTextureManager(), 2048,
				2048);
		legsTR = BitmapTextureAtlasTextureRegionFactory.createTiledFromAsset(
				legsTA, sm.activity.getAssets(), legSheet, 0, 0, L_COLUMN,
				L_ROWS);
		legsTA.load();
		
		backgroundTA = new BitmapTextureAtlas(sm.activity.getTextureManager(),
				2048, 2048);
		backgroundReg = BitmapTextureAtlasTextureRegionFactory
				.createFromAsset(backgroundTA, sm.activity,
						"background.png", 0, 0);
		backgroundTA.load();

		// close button
		closeButtonTA = new BitmapTextureAtlas(sm.activity.getTextureManager(),
				128, 64);
		closeButtonReg = BitmapTextureAtlasTextureRegionFactory
				.createFromAsset(closeButtonTA, sm.activity,
						"126x64_button_done.png", 0, 0);
		closeButtonTA.load();

		// font
		mFont = FontFactory.create(this.sm.engine.getFontManager(),
				this.sm.engine.getTextureManager(), 256, 256,
				Typeface.create(Typeface.DEFAULT, Typeface.BOLD), 20, true,
				Color.YELLOW);
		mFont.load();

	}

	@Override
	public Scene create() {
		Scene creatureScene = new Scene();
		client = new CCClient(this);
		//sm.activity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		int cX = (int)(sm.camera.getWidth() - backgroundReg.getWidth())/2;
		int cY = (int)(sm.camera.getHeight() - backgroundReg.getHeight())/2; 
		Sprite bgSprite = new Sprite(cX, cY,  backgroundReg, sm.activity.getVertexBufferObjectManager());
		bgSprite.setScaleX(sm.camera.getWidth()/bgSprite.getWidth());
		bgSprite.setScaleY(sm.camera.getHeight()/bgSprite.getHeight());
		SpriteBackground bg = new SpriteBackground(bgSprite);
		creatureScene.setBackground(bg);

		spritePart[1] = new AnimatedSprite(0, 0, bodysTR,
				sm.activity.getVertexBufferObjectManager()) {
			@Override
			public boolean onAreaTouched(final TouchEvent event,
					final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				if (event.getAction() == TouchEvent.ACTION_DOWN) {

					downx = this.getX() + pTouchAreaLocalX;
					downy = this.getY() + pTouchAreaLocalY;

				}

				return true;
			}
		};
		spritePart[1].setPosition(
				(sm.camera.getWidth() - spritePart[1].getWidth()) / 2,
				(sm.camera.getHeight() - spritePart[1].getHeight()) * 4 / 10);
		creatureScene.attachChild(spritePart[1]);
		creatureScene.registerTouchArea(spritePart[1]);

		float bodyYPosition = (sm.camera.getHeight() - spritePart[1]
				.getHeight()) * 4 / 10;

		spritePart[0] = new AnimatedSprite(0, 0, headsTR,
				sm.activity.getVertexBufferObjectManager()) {
			@Override
			public boolean onAreaTouched(final TouchEvent event,
					final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				if (event.getAction() == TouchEvent.ACTION_DOWN) {

					downx = this.getX() + pTouchAreaLocalX;
					downy = this.getY() + pTouchAreaLocalY;

				}

				return true;
			}
		};
		spritePart[0].setPosition(
				(sm.camera.getWidth() - spritePart[0].getWidth()) / 2,
				bodyYPosition - spritePart[0].getHeight());
		creatureScene.attachChild(spritePart[0]);
		// creatureScene.registerTouchArea(spritePart[0]);

		spritePart[2] = new AnimatedSprite(0, 0, legsTR,
				sm.activity.getVertexBufferObjectManager()) {
			@Override
			public boolean onAreaTouched(final TouchEvent event,
					final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				if (event.getAction() == TouchEvent.ACTION_DOWN) {

					downx = this.getX() + pTouchAreaLocalX;
					downy = this.getY() + pTouchAreaLocalY;

				}

				return true;
			}
		};
		spritePart[2].setPosition(
				(sm.camera.getWidth() - spritePart[2].getWidth()) / 2,
				bodyYPosition + spritePart[1].getHeight());
		creatureScene.attachChild(spritePart[2]);
		//creatureScene.registerTouchArea(spritePart[2]);

		// Timer text
		runTimer = true;
		timer = 20;
		timerText = new Text(sm.camera.getWidth() * 1 / 3,
				sm.camera.getHeight() * 7 / 8, mFont, "Time remaining: "
						+ timer, new TextOptions(HorizontalAlign.CENTER),
				this.sm.engine.getVertexBufferObjectManager());
		creatureScene.attachChild(timerText);

		// Timer Handler
		onTick();

		// close button

		/*Sprite closeButton = new Sprite(0, 0, closeButtonReg,
				sm.engine.getVertexBufferObjectManager()) {
			@Override
			public boolean onAreaTouched(final TouchEvent pSceneTouchEvent,
					final float pTouchAreaLocalX, final float pTouchAreaLocalY) {
				runTimer = false;
				// sm.setCurrentS(SceneManager.AllScenes.CWIN);
				selections[partId] = currentImg[partId];
				sm.setWinS(SceneManager.AllScenes.CWIN, selections);

				return true;
			}
		};
		closeButton.setPosition(
				(sm.camera.getWidth() * 4 / 5 - closeButton.getWidth() / 2),
				(sm.camera.getHeight() * 7 / 8 - closeButton.getHeight() / 2));
		creatureScene.attachChild(closeButton);
		creatureScene.registerTouchArea(closeButton);*/

		creatureScene.setOnSceneTouchListener(this);

		return creatureScene;
	}


	public void switchImages(float dx, float dy, float ux, float uy) {
		Log.d("before", "partId" + partId + "currentImg" + currentImg[partId]
				+ "max frame" + maxFrame[partId]);

		if (ux >= dx + 10) {

			if (currentImg[partId] > 1) {
				// spritePart[0].stopAnimation(currentBody -1);
				spritePart[partId].setCurrentTileIndex(--currentImg[partId]);
			} else {
				spritePart[partId].setCurrentTileIndex(maxFrame[partId]);
				currentImg[partId] = maxFrame[partId];
			}
		} else if (ux < dx - 10) {

			if (currentImg[partId] < maxFrame[partId]) {
				// spritePart[0].stopAnimation(currentBody + 1);
				spritePart[partId].setCurrentTileIndex(++currentImg[partId]);
			} else {
				spritePart[partId].setCurrentTileIndex(1);
				currentImg[partId] = 1;
			}
		}
		Log.d("after", "partId" + partId + "currentImg" + currentImg[partId]
				+ "max frame" + maxFrame[partId]);
		Packet0005Number mp = new Packet0005Number();
		mp.id = partId;
		mp.value = currentImg[partId];
		client.client.sendTCP(mp);

	}

	public void setImages(int id, int value) {

		selections[id] = value;
		//spritePart[id].setCurrentTileIndex(value);
		Log.i("switch", "id is: " + id + value);

	}

	public void setAssigenment(int index) {

		partId = index;
		spritePart[partId].setCurrentTileIndex(1);
		currentImg[partId] = 1;
		Log.i("Client", "The assignment is: " + index);

	}

	public void onTick() {
		if (runTimer) {
			sm.engine.registerUpdateHandler(new TimerHandler(1f,
					new ITimerCallback() {
						public void onTimePassed(TimerHandler pTimerHandler) {

							timerText.setText("Time remaining: " + timer);
							if (timer == 5) {
								timerText.setColor(0, 255, 0);
							}
							if (timer >= 0) {
								timer = timer - 1;
								pTimerHandler.reset();
							}
							if (timer < 0) {
								sm.engine
										.unregisterUpdateHandler(pTimerHandler);
								runTimer = false;
								//sm.setCurrentS(SceneManager.AllScenes.MENU);
								selections[partId] = currentImg[partId];
								sm.setWinS(SceneManager.AllScenes.CWIN, selections);
							}
						}
					}));
		}
	}

	@Override
	public boolean onSceneTouchEvent(Scene pScene, TouchEvent pSceneTouchEvent) {
		if (pSceneTouchEvent.getAction() == TouchEvent.ACTION_DOWN) {

			downx = pSceneTouchEvent.getX();
			downy = pSceneTouchEvent.getY();
			Log.d("touch", "down");

		} else if (!(downx == -1 || downy == -1)) {
			if (pSceneTouchEvent.getAction() == TouchEvent.ACTION_UP) {
				Log.d("touch", "up");
				float upx = pSceneTouchEvent.getX();
				float upy = pSceneTouchEvent.getY();
				switchImages(downx, downy, upx, upy);
				downx = -1;
				downy = -1;
			}
		}
		return true;
	}
	
	public void setTimer(){
		timer = 20;
	}

	// Networking component
	
	public void joinGame(int gameID) {
		Packet0006AssignParts pidP = new Packet0006AssignParts();
		pidP.number = gameID;
		client.client.sendTCP(pidP);
	}

}
