package database;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.LinkedList;

public class MySQLAccess {
	private static Connection connect = null;
	
	/**
	 * connects to the db
	 * @throws Exception
	 */
	public static void connectToDB() throws Exception {		
		try {
			//connects to the db with the server account
			Class.forName("com.mysql.jdbc.Driver");
			connect = DriverManager.getConnection(
					"jdbc:mysql://localhost/TrainDB?" 
			+ "user=server&password=serverpw");			
		}
		catch (Exception e) { 			
		}
	}
	
	/**
	 * close the db connection
	 */
	public static void close() {
		try {			
			if (connect != null) {
				connect.close();
			}
		} catch (Exception e) {			
		}
	}
	
	/**
	 * Add a new user account
	 * @param username
	 * @param password
	 * @return 
	 */
	public static String storeUser(String username, String password) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		String UID = null;
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//check if user exists already
			pStatement = connect.prepareStatement(
					"select * from users where username = ?");
			pStatement.setString(1, username);
			resultSet = pStatement.executeQuery();
			System.out.println("check if user exists already");
			//insert new user				
			if (!resultSet.next()) { //returns false if username not found
				System.out.println("Success. User not found. Add new user.");
				pStatement = connect.prepareStatement(
						"insert into users (username, password) values (?, ?)");
				pStatement.setString(1, username);
				pStatement.setString(2, password);		
				pStatement.executeUpdate();
				
				//get UID
				pStatement = connect.prepareStatement(
						"select UID from users where username = ?");
				pStatement.setString(1, username);
				System.out.println("getting UID");
				resultSet = pStatement.executeQuery();
				if (resultSet.next()) {
					UID = resultSet.getString(1);	
					System.out.println("uid is " + UID);
				}
				else { //error!!
					UID = "-1"; 
				}				
			}
			
			else { //error!!
				UID = "-1"; 
			}
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {
			System.out.println(e);
			return "-2";
		}		
		
		return UID;
	}
	
	/**
	 * updates a user's password
	 * @param UID
	 * @param password
	 */
	public static String updatePW(String UID, String password) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		String returnVal = "0";
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//check if user exists already
			pStatement = connect.prepareStatement(
					"select * from users where UID = ?");
			pStatement.setString(1, UID);
			resultSet = pStatement.executeQuery();
			
			//insert new user				
			if (resultSet.next()) { //returns false if username not found				
				pStatement = connect.prepareStatement(
						"update users set password = ? where UID = ?");
				pStatement.setString(1, password);
				pStatement.setString(2, UID);		
				pStatement.executeUpdate();
			}
			else { //error
				returnVal = "-1";
			}		
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}		
			
		} 
		catch (Exception e) {
			return "-1";
		}	
		return returnVal;
	}
	
	/**
	 * updates a user's password
	 * @param UID
	 * @param password
	 */
	public static String updateUserScore(String UID, String user_score) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		String returnVal = "0";
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//check if user exists already
			pStatement = connect.prepareStatement(
					"select * from users where UID = ?");
			pStatement.setString(1, UID);
			resultSet = pStatement.executeQuery();
			
			//update userscore				
			if (resultSet.next()) { //returns false if username not found				
				pStatement = connect.prepareStatement(
						"update users set user_score = ? where UID = ?");
				pStatement.setString(1, user_score);
				pStatement.setString(2, UID);		
				pStatement.executeUpdate();
			}
			else returnVal = "-1";
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {	
			return "-1";
		}		
		return returnVal;
	}
	
	/**
	 * updates a user's password
	 * @param UID
	 * @param password
	 */
	public static String updateCurrTrainScore(String UID, String curr_train_score) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		String returnVal = "0";
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//check if user exists already
			pStatement = connect.prepareStatement(
					"select * from users where UID = ?");
			pStatement.setString(1, UID);
			resultSet = pStatement.executeQuery();
			
			//update current train score				
			if (resultSet.next()) { //returns false if username not found				
				pStatement = connect.prepareStatement(
						"update users set current_train_score = ? where UID = ?");
				pStatement.setString(1, curr_train_score);
				pStatement.setString(2, UID);		
				pStatement.executeUpdate();
			}
			else {
				returnVal = "-1";
			}
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {	
			return "-1";
		}		
		return returnVal;
	}
	
	
	
	/**
	 * When two users play a minigame, 
	 * increment their playcount
	 * works with both new pairs and existing pairs of users
	 * @param UID1
	 * @param UID2
	 */
	public static void updatePlays(String UID1, String UID2) {
		PreparedStatement pStatement;			
		ResultSet resultSet;		
		String UIDHash = UID_Hash(UID1, UID2);
				
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//check if pair exists already
			pStatement = connect.prepareStatement(
					"select * from played_with where UID_HASH = ?");
			pStatement.setString(1, UIDHash);
			resultSet = pStatement.executeQuery();
			
			//update pair				
			if (!resultSet.next()) { //pair not found; add it!			
				pStatement = connect.prepareStatement(
						"insert into played_with (UID_HASH, plays) values (?, 1)");
				pStatement.setString(1, UIDHash);						
				pStatement.executeUpdate();
			}
			else { //pair found, simply update		
				pStatement = connect.prepareStatement(
						"update played_with set plays = plays + 1 where UID_HASH = ?");
				pStatement.setString(1, UIDHash);					
				pStatement.executeUpdate();
			}
						
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {						
		}
	}
	
	/**
	 * When two users trade cargo, 
	 * increment their cargo trade count
	 * works with both new pairs and existing pairs of users
	 * @param UID1
	 * @param UID2
	 */
	public static void updateCargoTrades(String UID1, String UID2) {
		PreparedStatement pStatement;			
		ResultSet resultSet;		
		String UIDHash = UID_Hash(UID1, UID2);
				
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//check if pair exists already
			pStatement = connect.prepareStatement(
					"select * from played_with where UID_HASH = ?");
			pStatement.setString(1, UIDHash);
			resultSet = pStatement.executeQuery();
			
			//update pair				
			if (!resultSet.next()) { //pair not found; add it!			
				pStatement = connect.prepareStatement(
						"insert into played_with (UID_HASH, cargo_trades) values (?, 1)");
				pStatement.setString(1, UIDHash);					
				pStatement.executeUpdate();
			}
			else { //pair found, simply update		
				pStatement = connect.prepareStatement(
						"update played_with set cargo_trades = cargo_trades + 1 where UID_HASH = ?");
				pStatement.setString(1, UIDHash);					
				pStatement.executeUpdate();
			}
						
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {						
		}
	}	
	
	/**
	 * Store a game score
	 * @param UID
	 * @param GID
	 * @param score
	 */
	public static void storeScore(String UID, String GID, String score) {
		PreparedStatement pStatement;		
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}			
			
			//insert new score					
			pStatement = connect.prepareStatement(
					"insert into scores (UID, GID, score) values (?, ?, ?)");
			pStatement.setString(1, UID);
			pStatement.setString(2, GID);
			pStatement.setString(3, score);
			pStatement.executeUpdate();			
			
			//close everything when done			
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {						
		}
	}
	
	/**
	 * Remove a user account
	 * @param username
	 * @param password
	 */
	public static String removeUser(String UID) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		String returnVal = "0";
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//check if user exists
			pStatement = connect.prepareStatement(
					"select * from users where UID = ?");
			pStatement.setString(1, UID);
			resultSet = pStatement.executeQuery();
			
			//remove user from all tables				
			if (resultSet.next()) { //returns false if user not found	
				//delete user
				pStatement = connect.prepareStatement(
						"delete from users where UID = ?");
				pStatement.setString(1, UID);						
				pStatement.executeUpdate();
				//delete played with				
				pStatement = connect.prepareStatement(
						"delete from played_with where UID_HASH like '?____' or UID_HASH like '____?';");
				pStatement.setString(1, UID);
				pStatement.setString(2, UID);
				pStatement.executeUpdate();
				//delete scores
				pStatement = connect.prepareStatement(
						"delete from scores where UID = ?");
				pStatement.setString(1, UID);						
				pStatement.executeUpdate();				
			}
			else {
				returnVal = "-1";
			}
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {	
			return "-1";
		}		
		return returnVal;
	}
	
	public static String getUsername(String UID) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		String username = null;
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//get user
			pStatement = connect.prepareStatement(
					"select username from users where UID = ?");
			pStatement.setString(1, UID);
			resultSet = pStatement.executeQuery();
			
			if(resultSet.next()) {
				username = resultSet.getString(1);
			}
			else {
				username = "-1";
			}
			
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {	
			return "-2";
		}		
		
		return username;
	}
	
	public static String getUserScore(String UID) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		String userscore = null;
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//get user
			pStatement = connect.prepareStatement(
					"select user_score from users where UID = ?");
			pStatement.setString(1, UID);
			resultSet = pStatement.executeQuery();
			
			if(resultSet.next()) {
				userscore = resultSet.getString(1);
			}
			else {
				userscore = "-1";
			}
			
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {	
			return "-1";
		}		
		
		return userscore;
	}
	
	public static String getCurrTrainScore(String UID) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		String curr_train_score = null;
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//get user
			pStatement = connect.prepareStatement(
					"select current_train_score from users where UID = ?");
			pStatement.setString(1, UID);
			resultSet = pStatement.executeQuery();
			
			if(resultSet.next()) {
				curr_train_score = resultSet.getString(1);
			}
			else {
				curr_train_score = "-1";
			}
			
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {	
			return "-1";
		}		
		
		return curr_train_score;
	}
	
	public static LinkedList<Pair<String, String>> getHighscores(String UID) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		LinkedList<Pair<String, String>> scores = new LinkedList<Pair<String, String>>();
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//get scores
			pStatement = connect.prepareStatement(
					"select GID, score from scores where UID = ?");
			pStatement.setString(1, UID);
			resultSet = pStatement.executeQuery();
			
			while(resultSet.next()) {
				String GID = resultSet.getString(1);
				String score = resultSet.getString(2);
				scores.add(new Pair<String, String>(GID, score));
			}
			if (scores.isEmpty()) {
				scores = null;
			}
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {	
			return null;
		}		
		
		return scores;
	}
	
	public static String getPlays(String UID1, String UID2) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		String num_plays = null;
		String UIDHash = UID_Hash(UID1, UID2);
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//get plays
			pStatement = connect.prepareStatement(
					"select plays from played_with where UID_HASH = ?");
			pStatement.setString(1, UIDHash);
			resultSet = pStatement.executeQuery();
			
			if(resultSet.next()) {
				num_plays = resultSet.getString(1);
			}
			else {
				num_plays = "0";
			}			
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {	
			return "-1";
		}		
		
		return num_plays;
	}
	
	public static String getCargoTrades(String UID1, String UID2) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		String num_cargo_trades = null;
		String UIDHash = UID_Hash(UID1, UID2);
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//get plays
			pStatement = connect.prepareStatement(
					"select cargo_trades from played_with where UID_HASH = ?");
			pStatement.setString(1, UIDHash);
			resultSet = pStatement.executeQuery();
			
			if(resultSet.next()) {
				num_cargo_trades = resultSet.getString(1);
			}
			else {
				num_cargo_trades = "-1";
			}
			
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {	
			return "-1";
		}		
		
		return num_cargo_trades;
	}
	
	public static String login(String username, String password) {
		PreparedStatement pStatement;		
		ResultSet resultSet;
		String UID = null;
		
		try {
			//check connection
			Boolean dbclosed = connect.isClosed();		
			while (connect == null || dbclosed) {			
				connectToDB();			
			}
			
			//check if user exists already
			pStatement = connect.prepareStatement(
					"select * from users where username = ? and password = ?");
			pStatement.setString(1, username);
			pStatement.setString(2, password);
			resultSet = pStatement.executeQuery();
			
			//insert new user				
			if (resultSet.next()) {				
				UID = resultSet.getString(1);			
			}
			
			else { //error!!
				UID = "-1"; 
			}
			
			//close everything when done
			if (resultSet != null) {
				resultSet.close();
			}
			if (pStatement != null) {
				pStatement.close();
			}					
		} 
		catch (Exception e) {	
			return "-1";
		}		
		
		return UID;
	}
	
	/**
	 * Takes two UIDs and forms their lookup hash used in played_with
	 * @param UID1
	 * @param UID2
	 * @return
	 */
	private static String UID_Hash(String UID1, String UID2) {
		String UIDHash;
		if (UID1.compareTo(UID2) < 0) {
			UIDHash = UID1 + UID2;
		}
		else if (UID1.compareTo(UID2) > 0) {
			UIDHash = UID2 + UID1;
		}
		else {
			//TODO: strings are equal. return error
			return null;
		}
		return UIDHash;
	}
}
