package events;

import java.util.Hashtable;
import java.util.Random;
import persistantgame.TrainStats;

public class EventsManager {
	private Hashtable<Integer, Event> eventList;
	Random rand;
	private static EventsManager instance = new EventsManager();
	
	public EventsManager(){
		eventList = new Hashtable<Integer, Event>();
		rand = new Random();
		importEvent();
	}
	
	public static EventsManager getInstance(){
		return instance;
	}
	
	private void importEvent(){		
		// 00
		eventList.put(0, new Event("Short cut","You may pay 30 gold to take a secret portion of the track with no stops or detours. As a result at the end of the turn you will have traveled an additional 10 spaces."){
			@Override
			public void activate() {
				int result = errorcheck(TrainStats.getInstance().getMoney() - 30);
				TrainStats.getInstance().setMoney(result);
			}
			
			@Override
			public void deactivate() {			
				TrainStats.getInstance().setDistance(TrainStats.getInstance().getDistance() + 10);
			}
		});

		// 01
		eventList.put(1, new Event("Poorly maintained track","Someone was slacking slightly on track maintenance. At the end of the turn travel 2 fewer distance and lose an extra health."){
			@Override
			public void deactivate() {
				int result = errorcheck(TrainStats.getInstance().getDistance() - 2);
				int result2 = errorcheck(TrainStats.getInstance().getHealth() - 1);
				TrainStats.getInstance().setDistance(result);
				TrainStats.getInstance().setHealth(result2);
			}
		});

		// 02
		eventList.put(2, new Event("Strange animal","You see an unheard of species of animal on the side of the road, and know a scientist nearby who would meet up and buy it from you. You will get $50 at the beginning of your next visit to the store, but there is a 50 percent chance of the creature damaging the train for 5 health."){
			@Override
			public void activate() {				
				TrainStats.getInstance().setMoney(TrainStats.getInstance().getMoney() + 50);
			}
			
			@Override
			public void deactivate() {				
				rand.setSeed(System.nanoTime());
				if (rand.nextBoolean()){
					int result = errorcheck(TrainStats.getInstance().getHealth() - 5);
					TrainStats.getInstance().setHealth(result);
				}
			}
		});

		// 03
		eventList.put(3, new Event("Come On Baby, Hold Together","Your health does not decrease this turn."){
			int oldHealth;
			
			@Override
			public void activate() {				
				oldHealth = TrainStats.getInstance().getHealth();
			}
			
			@Override
			public void deactivate() {
				TrainStats.getInstance().setHealth(oldHealth);
			}
		});

		// 04
		eventList.put(4, new Event("Superheroes on the train","Famous Japanese superheroes are here! Everyone is distracted, including the driver. Go 3 fewer distance this turn."){
			@Override
			public void deactivate() {
				int result = errorcheck(TrainStats.getInstance().getDistance() - 3);
				TrainStats.getInstance().setDistance(result);
			}
		});

		// 05
		eventList.put(5, new Event("A kind donation","A grateful passenger decided to donate $50 dollars to your train for an excellent experience during their trip"){
			@Override
			public void activate() {				
				TrainStats.getInstance().setMoney(TrainStats.getInstance().getMoney() + 50);
			}
		});

		// 6
		eventList.put(6, new Event("Sheep crossing","Move three spaces less than usual."){
			@Override
			public void deactivate() {
				int result = errorcheck(TrainStats.getInstance().getDistance() - 3);
				TrainStats.getInstance().setDistance(result);
			}
		});
		
		// 7
		eventList.put(7, new Event("New Trains","The train company decided to put new train model in use. Gain 1 speed point."){
			@Override
			public void activate() {				
				TrainStats.getInstance().setSpeed(TrainStats.getInstance().getSpeed() + 1);
			}
		});
		
		// 8
		eventList.put(8, new Event("The legendary engineer","An engineer uses magic to repair your train. Gain 10 health."){
			@Override
			public void activate() {				
				TrainStats.getInstance().setHealth(TrainStats.getInstance().getHealth() + 10);
			}
		});
		
		// 9
		eventList.put(9, new Event("Thunderstruck","The power plant has been struck by lightning. Lose 3 speed points."){
			@Override
			public void activate() {
				int result = errorcheck(TrainStats.getInstance().getSpeed() - 3);
				TrainStats.getInstance().setSpeed(result);
			}
		});
		
		// 10
		eventList.put(10, new Event("Poltergeist Pillagers","Ghosts are on the train and they stole 6 cargo!"){
			@Override
			public void activate() {
				int result = errorcheck(TrainStats.getInstance().getCargo() - 6);
				TrainStats.getInstance().setCargo(result);
			}
		});
	}
	
	public Event getRandomEvent(){
		rand.setSeed(System.nanoTime());
		Object[] keys = eventList.keySet().toArray();
		Integer randomKey = (Integer) keys[rand.nextInt(keys.length)];
		
		return eventList.get(randomKey);
	}
}
