package lanceGameRoomNetworking;


import java.io.IOException;
import java.util.Hashtable;
import java.util.LinkedList;

import lanceGameRoomNetworking.packets.GameLogInRequest;
import lanceGameRoomNetworking.packets.GameRoomExitRequest;
import lanceGameRoomNetworking.packets.GameRoomInfo;
import lanceGameRoomNetworking.packets.GameRoomStartRequest;
import lanceGameRoomNetworking.packets.GameStartRequest;
import lanceGameRoomNetworking.packets.GameUpdateRequest;
import lanceGameRoomNetworking.packets.TrainLogInRequest;
import lanceGameRoomNetworking.packets.TrainRoomInfo;
import lanceGameRoomNetworking.packets.TrainUpdateRequest;

import networking.packets.*;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryonet.Connection;
import com.esotericsoftware.kryonet.Server;
import com.esotericsoftware.minlog.Log;

public class RoomServer {
	private Server server;
	public Hashtable<Integer, LinkedList<Connection>> cList;
	private int count;
	
	private TrainWorld trainworld;
	
	public RoomServer() throws IOException{
		server = new Server();
		
		trainworld = new TrainWorld();
		
		cList = new Hashtable<Integer, LinkedList<Connection>>();
		count = 0;
		registerPackets();
		//TODO
		RoomListenerS l = new RoomListenerS(this);
		server.addListener(l);
		server.bind(23333);
		System.out.println("ARRIVED HERE");
		server.start();
	}
	
	private void registerPackets(){
		Kryo kryo = server.getKryo();
		kryo.register(GameLogInRequest.class);
		kryo.register(GameRoomExitRequest.class);
		kryo.register(GameRoomInfo.class);
		kryo.register(GameRoomStartRequest.class);
		kryo.register(GameUpdateRequest.class);
		kryo.register(TrainLogInRequest.class);
		kryo.register(TrainRoomInfo.class);
		kryo.register(TrainUpdateRequest.class);
		kryo.register(GameStartRequest.class);
		kryo.register(char[].class);
		kryo.register(int[].class);
	}
	
	public void broadcastOnce(int gameroomid, char[] trainId){
		GameStartRequest packet = new GameStartRequest();
		packet.gameid = trainworld.retrieveTrainRoom(trainId).getGameRoom(gameroomid).getGameId();
		
		for (Connection c: cList.get(gameroomid)){
			c.sendTCP(packet);
		}
		cList.clear();
		count = 0;
	}

	public int getCount() {
		return count;
	}

	public void setCount(int count) {
		this.count = count;
	}

	public static void main(String[] args){
		try {
			new RoomServer();
			Log.set(Log.LEVEL_DEBUG);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public TrainWorld getTrainworld() {
		return trainworld;
	}

	public void setTrainworld(TrainWorld trainworld) {
		this.trainworld = trainworld;
	}
	
}
