package lanceGameRoomNetworking;


import java.util.LinkedList;

import lanceGameRoomNetworking.packets.TrainRoomInfo;

public class TrainRoom {
	private String trainId;
	private LinkedList<GameRoom> gamerooms;
	
	public TrainRoom(String trainId) {
		this.trainId = trainId;
		setGamerooms(new LinkedList<GameRoom>());
	}
	
	public TrainRoomInfo getInfo(){
		TrainRoomInfo result = new TrainRoomInfo();
		result.roomIds = new int[gamerooms.size()];
		result.gameIds = new int[gamerooms.size()];
		result.existingPlayers = new int[gamerooms.size()];
		result.numOfPlayers = new int[gamerooms.size()];
		
		int i = 0;
		
		for(GameRoom gr: gamerooms){
			result.roomIds[i] = gr.getGameroomid();
			result.gameIds[i] = gr.getGameId();
			result.existingPlayers[i] = gr.getExistingPlayers();
			result.numOfPlayers[i] = gr.getNumOfPlayers();
			i++;
		}
		
		return result;
	}
	
	// Return: the new game room id
	public int host(int gameid){
		int roomid = RoomIdGenerator.getInstance().generateGameRoomId();
		gamerooms.offer(new GameRoom(roomid, gameid, GameInfoManager.getInstance().retrieveMaxPlayers(gameid)));
		return roomid;
	}
	
	public GameRoom getGameRoom(int gameroomid){
		for (GameRoom g: gamerooms){
			if (g.getGameroomid() == gameroomid){
				return g;
			}
		}
		return null;
	}
	
	public void startGame(int gameroomid){
		GameRoom gr = getGameRoom(gameroomid);
		if (gr != null){
			// Start the game
			gr.start();
			// Remove the gameroom from the list
			gamerooms.remove(gr);
		}
	}

	public String getTrainId() {
		return trainId;
	}

	public void setTrainId(String trainId) {
		this.trainId = trainId;
	}
 
	public LinkedList<GameRoom> getGamerooms() {
		return gamerooms;
	}

	public void setGamerooms(LinkedList<GameRoom> gamerooms) {
		this.gamerooms = gamerooms;
	}
	
	@Override
	public String toString() {
		String result = "";
		for (GameRoom gr: gamerooms){
			result = result + GameInfoManager.getInstance().retrieveGameName(gr.getGameId()) + " " + gr.getExistingPlayers() + "/" + gr.getNumOfPlayers() + "$" ;
		}
		return result;
	}
}
