package mainmenu.networking;

import java.util.LinkedList;

import mainmenu.MainMenuActivity;
import networking.packets.Packet0007String;
import networking.packets.Packet0008Games;
import android.content.SharedPreferences;
import android.util.Log;
import android.widget.Toast;

import com.example.traingame.MainActivity;

import database.DBClient;

public class GameJoinProtocolC {
	private GameJoinClient client;

	public GameJoinProtocolC(GameJoinClient client) {
		this.client = client;
	}

	public void getGames() {
		LinkedList<LinkedList<Integer>> sendData = new LinkedList<LinkedList<Integer>>();
		LinkedList<Integer> header = new LinkedList<Integer>();
		header.add(0);
		LinkedList<Integer> sendDataInternal = new LinkedList<Integer>();		
		sendDataInternal.add(0);
		sendDataInternal.add(getMyUID());
		sendData.add(header);
		sendData.add(sendDataInternal);
		send(sendData);
	}

	public void joinGame(Integer gameID) {
		LinkedList<LinkedList<Integer>> sendData = new LinkedList<LinkedList<Integer>>();
		LinkedList<Integer> header = new LinkedList<Integer>();
		header.add(1);
		LinkedList<Integer> sendDataInternal = new LinkedList<Integer>();		
		sendDataInternal.add(gameID);
		sendDataInternal.add(getMyUID());
		sendData.add(header);
		sendData.add(sendDataInternal);
		send(sendData);
	}

	public void leaveGame(Integer gameID) {
		LinkedList<LinkedList<Integer>> sendData = new LinkedList<LinkedList<Integer>>();
		LinkedList<Integer> sendDataInternal = new LinkedList<Integer>();
		LinkedList<Integer> header = new LinkedList<Integer>();
		header.add(2);		
		sendDataInternal.add(gameID);
		sendDataInternal.add(getMyUID());
		sendData.add(header);
		sendData.add(sendDataInternal);
		send(sendData);
	}

	public void hostNewGame(Integer gameNameID) {
		LinkedList<LinkedList<Integer>> sendData = new LinkedList<LinkedList<Integer>>();
		LinkedList<Integer> sendDataInternal = new LinkedList<Integer>();
		LinkedList<Integer> header = new LinkedList<Integer>();
		header.add(3);		
		sendDataInternal.add(gameNameID);
		sendDataInternal.add(getMyUID());
		sendData.add(header);
		sendData.add(sendDataInternal);
		send(sendData);
	}

	public void hostStartsGame(Integer gameID) {
		LinkedList<LinkedList<Integer>> sendData = new LinkedList<LinkedList<Integer>>();
		LinkedList<Integer> sendDataInternal = new LinkedList<Integer>();
		LinkedList<Integer> header = new LinkedList<Integer>();
		header.add(4);		
		sendDataInternal.add(gameID);
		sendDataInternal.add(getMyUID());
		sendData.add(header);
		sendData.add(sendDataInternal);
		send(sendData);
	}

	public void send(LinkedList<LinkedList<Integer>> message) {
		for (Integer num : message.get(0)) {
			Log.d("testing", "sending "+ num);
		}
		Packet0008Games mp = new Packet0008Games();
		mp.games = message;
		this.client.client.sendTCP(mp);
		Log.d("testing", "Games sent");
	}
	
	public void joinTrainRoom(String trainRoom) {
		Packet0007String mp = new Packet0007String();		
		String message = String.format("%04d", Integer.valueOf(getMyUID())) + trainRoom;
		mp.message = message.toCharArray();
		this.client.client.sendTCP(mp);
		Log.d("gjs", "sent train room " + message);
	}
	
	public void getTrainRoomUsers(String trainRoom) {
		Packet0007String mp = new Packet0007String();		
		String message = "0000" + trainRoom;
		mp.message = message.toCharArray();
		this.client.client.sendTCP(mp);
		Log.d("gjs", "request for users sent " + message);
	}

	// RECEIVING
	public void handleRecievedGame(LinkedList<LinkedList<Integer>> message) {
		int header = message.get(0).get(0);
		int gameID;
		Log.d("testing", "something received! = " + header);
		if (header == -1) {
			Log.d("gjs", "error received");
			client.activity.runOnUiThread(new Runnable() {			
				@Override
				public void run() {	
					Toast.makeText(client.activity.getApplicationContext(), "Sorry, illegal action.", Toast.LENGTH_SHORT).show();
				}
			});			
		}
		else if (header == -2) { //users
			LinkedList<Integer> UIDs = message.get(0);
			Log.d("users", "user list size = " + UIDs.size());
			for (Integer num : UIDs) {
				Log.d("users", "User List contains " + num);
			}
			
			while (UIDs.get(0) == -2) {
				UIDs.remove(0);
			}
				
			Log.d("users", "user list final size = " + UIDs.size());
			MainMenuActivity.setUIDs(UIDs);
		}
		else if (header == -3) { //no games
			client.activity.updateGames();			
		} 		
		else if (header == -5) {	//leaving game action or join game failed			
			client.activity.leaveGameCompleteOrJoinGameFailure();
			client.activity.updateGames();
		}
		else if (header == -7) { //start game
			gameID = message.get(0).get(1);
			Log.d("gjs", "starting game " + gameID);
			this.client.startGame(gameID);
		} 
		else if (header == -8) { //remove game
			gameID = message.get(0).get(1);
			Log.d("gjs", "removing game " + gameID);
			client.removeGame(gameID);
		} 
		
		else {
			for (LinkedList<Integer> game : message) {
				Log.d("gjs", "updating game, position " + message.indexOf(game));
				updateGames(game);
			}
			client.activity.updateGames();
		}
	}

	public int getMyUID() {
		SharedPreferences sp = client.activity.getSharedPrefs();
		String uidStr = sp.getString("uid", "-1");		
		int UID = Integer.valueOf(uidStr);
		return UID;
	}

	public void updateGames(LinkedList<Integer> gameInts) {
		Integer gameID = gameInts.remove(0);
		Integer gameNameID = gameInts.remove(0);
		Integer timer = gameInts.remove(0);
		Integer host = gameInts.remove(0);		
		LinkedList<Integer> users = gameInts;		
		
		Game newGame = new Game(gameID, gameNameID, timer, host, users);
		Log.d("gjs", "new game made. ID = " + gameID + ", name = " + gameNameID + ", host = " + host + ", total users = " + users.size() + 1);
		client.games.put(gameID, newGame);	
		Log.d("gjs", "games hashmap size = " + client.games.size());
		//TODO: does the UI update now too?
	}
}
