package mainmenu.networking;

import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Timer;
import java.util.TimerTask;

import networking.packets.Packet0006Long;
import networking.packets.Packet0007String;
import networking.packets.Packet0008Games;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryonet.Connection;
import com.esotericsoftware.kryonet.Server;
import com.example.lctnetworktesting.networking.packets.Packet0000LogInfo;
import com.example.lctnetworktesting.networking.packets.Packet0001LogInAccepted;
import com.example.lctnetworktesting.networking.packets.Packet0002Message;
import com.example.lctnetworktesting.networking.packets.Packet0003SyncReq;
import com.example.lctnetworktesting.networking.packets.Packet0004Ack;
import com.example.lctnetworktesting.networking.packets.Packet0005Number;

public class GameJoinServer {
	private Server server;
	public LinkedList<Connection> cList;
	public LinkedList<Connection> cList2;
	HashMap<Integer, Game> games = new HashMap<Integer, Game>();	
	HashMap<Integer, GameData> gameData = new HashMap<Integer, GameData>();
	HashMap<Integer,ServerUserData> userData = new HashMap<Integer, ServerUserData>(); //Key = UID
	HashMap<String, LinkedList<Integer>> trainRooms = new HashMap<String, LinkedList<Integer>>();
	private int count = 0; 
	static int gameNum = 1; //used for gameID
	
	public GameJoinServer() throws IOException {
		server = new Server();

		cList = new LinkedList<Connection>();
		cList2 = new LinkedList<Connection>();		
		registerPackets();		
		GameJoinListenerS l = new GameJoinListenerS(this);
		server.addListener(l);
		server.bind(9001);
		server.start();
		
		gameData.put(1, new GameData("Bomb Pass Panic", 4, 6));
		gameData.put(2, new GameData("Elemancer", 1, 2));
		gameData.put(3, new GameData("Creature Creator", 3, 3));
		gameData.put(4, new GameData("Treasure Trap", 4, 4));	
	}

	private void registerPackets() {
		Kryo kryo = server.getKryo();
		kryo.register(Packet0000LogInfo.class);
		kryo.register(Packet0001LogInAccepted.class);
		kryo.register(Packet0002Message.class);
		kryo.register(Packet0003SyncReq.class);
		kryo.register(Packet0004Ack.class);
		kryo.register(Packet0005Number.class);
		kryo.register(Packet0006Long.class);
		kryo.register(Packet0007String.class);
		kryo.register(char[].class);
		kryo.register(LinkedList.class);
		kryo.register(Packet0008Games.class);			
	}

	public void broadcastOnce() {
		for (Connection c : cList) {
			Packet0002Message p = new Packet0002Message();
			p.message = "Two Person Play! Size is " + cList.size();
			c.sendTCP(p);
		}
		cList.clear();
		count = 0;
	}

	public void broadcastAgain() {
		Packet0002Message p = new Packet0002Message();
		p.message = "Two Person Play! Size is " + cList.size();
		server.sendToAllTCP(p);
	}

	public int getCount() {
		return count;
	}

	public void setCount(int count) {
		this.count = count;
	}
	
	/**
	 * Given a list of UIDs, returns all their connections
	 * used for broadcasting purposes
	 * @param UIDs
	 * @return
	 */
	public LinkedList<Connection> getConnections(LinkedList<Integer> UIDs){
		LinkedList<Connection> cons = new LinkedList<Connection>();
		for (Integer UID : UIDs) {
			if (userData.containsKey(UID)) {
				Connection con = userData.get(UID).connection;
				if (con != null) {
					cons.add(con);
				}
			}
		}
		return cons;
	}
	
	/**
	 * Given a game id, returns all their connections
	 * used for broadcasting purposes
	 * @param UIDs
	 * @return
	 */
	public LinkedList<Connection> getConnections(Integer gameID){
		Game game = this.games.get(gameID);
		LinkedList<Integer> UIDs = game.userList;
		UIDs.add(game.host);
		System.out.println("first uid in getConnections = " + UIDs.get(0));
		return getConnections(UIDs);
	}

	public LinkedList<Connection> getRoomConnections(String trainroom){
		LinkedList<Integer> users = trainRooms.get(trainroom);
		return getConnections(users);
	}
	
	public int nextKey() {
		int temp = gameNum;
		gameNum = gameNum + 1;
		System.out.println("GAMENUM UPDATED TO " + gameNum);
		return temp;
	}
	
	public static void main(String [] args){
		
		try {
			new GameJoinServer();
		} catch (IOException e) {			
			e.printStackTrace();
		}
	}

	public void removeGame(Integer gameID) {
		LinkedList<Connection> connections = new LinkedList<Connection>();
		LinkedList<LinkedList<Integer>> sendContents = new LinkedList<LinkedList<Integer>>();
		Game game = this.games.get(gameID);
		String trainRoom = game.trainRoom;
		LinkedList<Integer> UIDs = game.userList;
		UIDs.add(game.host);
		
		for (Integer UID : UIDs) {
			if (userData.containsKey(UID)) {
				ServerUserData userInfo = userData.get(UID);
				userInfo.gameID = -1;
				userData.put(UID, userInfo);
			}
		}
		
		this.games.remove(gameID);
		System.out.println("removing game " + gameID);
		System.out.println("HASHMAP SIZE IS NOW " + games.size());
		connections = this.getRoomConnections(trainRoom);
		LinkedList<Integer> removeGame = new LinkedList<Integer>();
		removeGame.add(-8);
		removeGame.add(gameID);
		sendContents.add(removeGame);
		GameJoinProtocolS.send(connections, sendContents);
		
	}

	public void startGame(Integer gameID) {	
		LinkedList<Connection> connections = new LinkedList<Connection>();
		LinkedList<LinkedList<Integer>> sendContents = new LinkedList<LinkedList<Integer>>();		
		connections = this.getConnections(gameID);
		System.out.println("StartGame num connections = " + connections.size());
		LinkedList<Integer> startGame = new LinkedList<Integer>();
		startGame.add(-7);
		startGame.add(gameID);
		sendContents.add(startGame);
		System.out.println("StartGame starting " + gameID);
		GameJoinProtocolS.send(connections, sendContents);
		removeGame(gameID);
	}
	
	public void setGameTimer(Integer gameID) {
		System.out.println("in setTimer");
		final Integer GID = gameID;
		Timer timer = new Timer();
		timer.schedule(new TimerTask() {
			@Override
			public void run() {
				System.out.println("running StartGame for "+ GID);
				startGame(GID);
			}			
		}, 20*1000); //20 seconds
	}
}
