package viewUsers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import mainmenu.MainMenuActivity;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;

import android.widget.Toast;
import com.example.traingame.R;

import database.DBProtocolC;

public class ViewUsersActivity extends Activity{	
	private Integer myUID;
	private ListView listview;
	private ArrayAdapter<String> adapter;
	private static LinkedList<Integer> UIDs = new LinkedList<Integer>();
	private ArrayList<String> usernames = new ArrayList<String>();
	private String trainRoom;
	private HashMap<Integer, OtherUser> userData;

	@SuppressLint("NewApi")
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_view_users);
		
		if (getIntent().getExtras() != null) {
			myUID = getIntent().getExtras().getInt("myUID");
			trainRoom = getIntent().getExtras().getString("trainRoom", "No Room Found");
			int[] uidArray = getIntent().getExtras().getIntArray("UIDs");
			if (uidArray != null) {
				UIDs.clear();
				for (int uid : uidArray) {
					UIDs.add(uid);
					Log.d("users", "in Users, got UID " + uid);
				}
			}
		}
		
		userData = MainMenuActivity.getUsers();
		
		//Set up adapter
		listview = (ListView) findViewById(R.id.view_users_listview);					
		adapter = new ArrayAdapter<String>(this, R.layout.list_all_users, new ArrayList<String>());
		listview.setAdapter(adapter);	
	    listview.setClickable(true);
	    listview.setOnItemClickListener(new AdapterView.OnItemClickListener() {
	    	  @Override
	    	  public void onItemClick(AdapterView<?> arg0, final View arg1, int position, long id) {
	    		  Log.d("testing", "list item clicked! pos " + position);
	    		  if (position < UIDs.size()) {
		    		  Integer UID = UIDs.get(position);
		    		  switchToViewSingleUser(UID);		    		  
	    		  }
	    		  else {
	    			  Toast.makeText(getApplicationContext(), "Sorry, bug. Click another user.", Toast.LENGTH_SHORT).show();
	    		  }
	    	  }			
    	});
		
		new getUsernamesTask().execute(this); 
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}	


	protected void switchToViewSingleUser(Integer UID) {		
		new getUserDataTask().execute(UID);
	}
	
	protected void switchToViewSingleUser(OtherUser user) {		
		Intent intent = new Intent(this, ViewSingleUserActivity.class);
		intent.putExtra("username", user.getUsername());	
		intent.putExtra("userscore", user.getUser_score());
		intent.putExtra("plays", user.getPlays());
		intent.putExtra("train", user.getCurr_train_score());
		startActivity(intent);
	}

	private class getUsernamesTask extends AsyncTask<ViewUsersActivity, Void, Void>{		
		
		@Override
		protected Void doInBackground(ViewUsersActivity... params) {			
			DBProtocolC.connectToDB();			
			String username;
			for (Integer UID : UIDs) {
  			  if (!userData.containsKey(UID) || userData.get(UID).getUsername() == null) {
  				username = DBProtocolC.getUsername(UID);
  				String playcount = DBProtocolC.getPlays(UID, myUID);
  				userData.put(UID, new OtherUser(UID, username, playcount));
  			  }
  			  else {
  				username = userData.get(UID).getUsername();
  			  }  			  
  			  String displayString = username + "    -    " + userData.get(UID).getPlays() + " Plays";  			  
  			  usernames.add(displayString);
			}
			DBProtocolC.close(); 			
			
		    MainMenuActivity.setUsers(userData);
		    updateUsernames();
			return null;	
		}
	}
	
	private class getUserDataTask extends AsyncTask<Integer, Void, Void>{	
		
		
		@Override
		protected Void doInBackground(Integer... params) {
			Integer UID = params[0];
			OtherUser user = userData.get(params[0]);
			
			if (user.getUser_score() == null || user.getCurr_train_score() == null) {
				DBProtocolC.connectToDB();
				user.setUser_score(DBProtocolC.getUserScore(String.format("%04d", UID)));
				user.setCurr_train_score(DBProtocolC.getCurrTrainScore(String.format("%04d", UID)));				
				DBProtocolC.close(); 
				userData.put(UID, user);
				MainMenuActivity.setUsers(userData);
			}
			switchToViewSingleUser(user);
			return null;	
		}
	}
	
	public void updateUsernames() {
		this.runOnUiThread(new Runnable() {			
			@SuppressLint("NewApi")
			@Override
			public void run() {	
				Log.d("users", "UPDATE usernames CALLED");			
				adapter.addAll(usernames);								
				adapter.notifyDataSetChanged();	
				Log.d("users", "adapter size is " + adapter.getCount());						
			}
		});
		
	}
	
	@Override 
	public void onStop() {
		super.onStop();
		finish();
	}
}
